﻿using System;
using System.ServiceModel;
using VisualComponents.UX.Shared;
using VisualComponents.Create3D;
using Caliburn.Micro;
using System.ComponentModel.Composition;
using System.Collections.Generic;
using System.Linq;

namespace VC.WcfServiceDemo
{
    /// <summary>
    /// Implementation of IVCService
    /// </summary>
    [Export(typeof(IPlugin))]
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.PerSession, ConcurrencyMode = ConcurrencyMode.Multiple)]
    public class VCService : IVCService
    {
        IApplication _Application;
        ISelectionManager _SelectionManager;
        ISignal _Signal;
        #region IPlugin implementation

        public void Initialize()
        {
            _SelectionManager = IoC.Get<ISelectionManager>();
            _Application = IoC.Get<IApplication>();
            _Application.World.ComponentAdded += World_ComponentAdded;
            _Application.World.ComponentRemoving += World_ComponentRemoving;
            _SelectionManager.SelectionChanged += VCService_SelectionChanged;
            _Signal = null;
        }

        public void Exit()
        {

        }

        #endregion IPlugin Implementation


        private void VCWCFCallbackCaller(Action<IVCServiceCallback> action)
        {
            foreach (IVCServiceCallback subscribedClient in VCServiceCallback.GetInstance().subscribedClients)
            {
                IVCServiceCallback callback = subscribedClient;
                try
                {
                    action(callback);
                }
                catch (ObjectDisposedException)
                {
                    VCServiceCallback.GetInstance().subscribedClients.Remove(callback);
                }
            }
        }

        #region Callbacks
        private void VCService_SelectionChanged(object sender, EventArgs e)
        {
            try
            {
                var selectedComponent = IoC.Get<ISelectionManager>().GetSelected<ISimComponent>();

                if (info.Count > 0)
                {
                    VCWCFCallbackCaller(delegate (IVCServiceCallback callback)
                    {
                        callback.GenericCallback(info);
                    });
                }

                if (selectedComponent != null)
                {
                    VCWCFCallbackCaller(delegate (IVCServiceCallback callback)
                    {
                        callback.GenericCallback2(selectedComponent.Name);
                    });

                    IBehavior beh = selectedComponent.GetBehavior(BehaviorType.BooleanSignal);

                    if (beh != null)
                    {
                        if (_Signal == null)
                        {
                            _Signal = beh as ISignal;
                            _Signal.SignalTrigger += Signal_SignalTrigger;
                        }
                        else
                        {
                            _Signal.SignalTrigger -= Signal_SignalTrigger;
                            _Signal = null;
                            _Signal = beh as ISignal;
                            _Signal.SignalTrigger += Signal_SignalTrigger;
                        }
                    }
                    else
                    {
                        if (_Signal != null)
                        {
                            _Signal.SignalTrigger -= Signal_SignalTrigger;
                            _Signal = null;
                        }
                    }
                }
                else
                {
                    if (_Signal != null)
                    {
                        _Signal.SignalTrigger -= Signal_SignalTrigger;
                        _Signal = null;
                    }


                    VCWCFCallbackCaller(delegate (IVCServiceCallback callback)
                    {
                        callback.GenericCallback2(string.Empty);
                    });
                }
            }
            catch
            {

            }
        }


        List<string> info = new List<string>();
        private void World_ComponentAdded(object sender, ComponentAddedEventArgs e)
        {
            info.Add(e.Component.Name);
            VCWCFCallbackCaller(delegate (IVCServiceCallback callback)
            {
                callback.GenericCallback(info);
            });
        }


        private void World_ComponentRemoving(object sender, ComponentRemovingEventArgs e)
        {
            info.Remove(e.Component.Name);
            VCWCFCallbackCaller(delegate (IVCServiceCallback callback)
            {
                callback.GenericCallback(info);
            });
        }

        private void Signal_SignalTrigger(object sender, SignalTriggerEventArgs e)
        {
            bool signalValue = (bool)e.Signal.Value;
            string signalName = e.Signal.Name;
            string component = e.Signal.Node.Component.Name;

            VCWCFCallbackCaller(delegate (IVCServiceCallback callback)
            {
                callback.OnSignal(signalValue, signalName, component);
            });
        }

        #endregion Callbacks


        #region IVCService implementation

        public bool Subscribe()
        {
            try
            {
                IVCServiceCallback subscribedClient = OperationContext.Current.GetCallbackChannel<IVCServiceCallback>();

                if (!VCServiceCallback.GetInstance().subscribedClients.Contains(subscribedClient))
                {
                    VCServiceCallback.GetInstance().subscribedClients.Add(subscribedClient);
                }
                return true;
            }
            catch
            {
                return false;
            }
        }

        public bool UnSubscribe()
        {
            try
            {
                IVCServiceCallback callback = OperationContext.Current.GetCallbackChannel<IVCServiceCallback>();
                if (VCServiceCallback.GetInstance().subscribedClients.Contains(callback))
                    VCServiceCallback.GetInstance().subscribedClients.Remove(callback);
                return true;
            }
            catch
            {
                return false;
            }
        }

        public void LoadLayout(string LayoutUri)
        {
            IoC.Get<IApplication>().LoadLayout(new Uri(LayoutUri));
        }

        public void SelectComponent(string Name)
        {
            var components = IoC.Get<IApplication>().World.Components;

            foreach (ISimComponent comp in components.Where(i => i.Name == Name))
            {
                IoC.Get<ISelectionManager>().SetSelection(comp);
            }
        }

        #endregion IVCService implementation
    }
}
